@@ -1,8 +1,8 @@
-from examples import acquire_token_client_credentials
+from examples import acquire_token_by_client_credentials
 from office365.graph_client import GraphClient
 from tests import test_user_principal_name, test_user_principal_name_alt
 
-client = GraphClient(acquire_token_client_credentials)
+client = GraphClient(acquire_token_by_client_credentials)
 message_json = {
     "Message": {
         "Subject": "Meet for lunch?",
@@ -20,6 +20,5 @@ message_json = {
     },
     "SaveToSentItems": "false"
 }
-
-client.users[test_user_principal_name].send_mail(message_json)
-client.execute_query()
+target_user = client.users[test_user_principal_name]
+target_user.send_mail(message_json).execute_query()
