@@ -1,4 +1,5 @@
 from office365.runtime.client_object_collection import ClientObjectCollection
+from office365.runtime.queries.create_entity_query import CreateEntityQuery
 from office365.runtime.resource_path import ResourcePath
 
 
@@ -16,6 +17,18 @@ class EntityCollection(ClientObjectCollection):
             return super(EntityCollection, self).__getitem__(key)
         return self._item_type(self.context, ResourcePath(key, self.resource_path))
 
+    def add_from_json(self, params):
+        """Creates a Entity resource from JSON
+
+        :type params: dict
+        :rtype: office365.entity.Entity
+        """
+        entity_object = self._item_type(self.context)
+        self.add_child(entity_object)
+        qry = CreateEntityQuery(self, params, entity_object)
+        self.context.add_query(qry)
+        return entity_object
+
     @property
     def context(self):
         """
