@@ -1,9 +1,9 @@
+from office365.entity_collection import EntityCollection
 from office365.onedrive.site import Site
-from office365.runtime.client_object_collection import ClientObjectCollection
 from office365.runtime.resource_path import ResourcePath
 
 
-class SiteCollection(ClientObjectCollection):
+class SiteCollection(EntityCollection):
     """Drive site's collection"""
 
     def __init__(self, context, resource_path=None):
@@ -12,7 +12,8 @@ class SiteCollection(ClientObjectCollection):
     @property
     def root(self):
         """If present, indicates that this is a root site collection in SharePoint."""
-        if self.is_property_available('root'):
-            return self.properties['root']
-        else:
-            return Site(self.context, ResourcePath("root", self.resource_path))
+        root_site = self.properties.get('root',
+                                        Site(self.context, ResourcePath("root", self.resource_path)))
+
+        root_site.ensure_property("id")
+        return root_site
