@@ -5,42 +5,45 @@ from office365.runtime.queries.batch_query import BatchQuery
 
 
 class ODataV4BatchRequest(ClientRequest):
+    """
+
+    """
 
     def __init__(self, context):
         super().__init__(context)
 
-    def build_request(self):
+    def build_request(self, query):
+        """
+
+        :type query: BatchQuery
+        """
         url = "{0}$batch".format(self.context.service_root_url())
         request = RequestOptions(url)
         request.method = HttpMethod.Post
         request.ensure_header('Content-Type', "application/json")
         request.ensure_header('Accept', "application/json")
-        request.data = self._prepare_payload()
+        request.data = self._prepare_payload(query)
         return request
 
-    def process_response(self, response):
+    def process_response(self, response, query):
         """Parses an HTTP response.
 
         :type response: requests.Response
+        :type query: office365.runtime.queries.client_query.BatchQuery
         """
         json = response.json()
         for resp in json["responses"]:
-            sub_qry = self._current_query.get(int(resp["id"]))
+            sub_qry = query.get(int(resp["id"]))
             self.context.pending_request().map_json(resp["body"], sub_qry.return_type)
 
-    def next_query(self):
-        queries = [qry for qry in self.context.pending_request().next_query()]
-        if len(queries) > 0:
-            batch_qry = BatchQuery(self.context, queries)  # Aggregate requests into batch request
-            self._current_query = batch_qry
-            yield batch_qry
-
-    def _prepare_payload(self):
+    def _prepare_payload(self, query):
         """Serializes a batch request body.
+
+        :type query: office365.runtime.queries.client_query.BatchQuery
         """
 
         requests_json = []
-        for qry in self._current_query.queries:
+        for qry in query.queries:
             request = qry.build_request()
             requests_json.append(self._serialize_request(request, len(requests_json)))
 
