@@ -1,4 +1,5 @@
 from office365.runtime.client_object import ClientObject
+from office365.runtime.queries.delete_entity_query import DeleteEntityQuery
 from office365.runtime.queries.update_entity_query import UpdateEntityQuery
 
 
@@ -6,21 +7,30 @@ class BaseEntity(ClientObject):
 
     def __init__(self, context, resource_path=None, namespace="SP", parent_collection=None):
         """
-        SharePoint entity
+        SharePoint specific entity
 
         :param office365.sharepoint.client_context.ClientContext context: SharePoint context
         :param ResourcePath resource_path: Resource Path
         :param str namespace:  default namespace
         """
-        super().__init__(context, resource_path, None, parent_collection)
-        self._namespace = namespace
+        super().__init__(context, resource_path, parent_collection, namespace)
 
     def with_credentials(self, credentials):
+        """
+        :type credentials:  UserCredential or ClientCredential
+        """
         self.context.with_credentials(credentials)
         return self
 
+    def delete_object(self):
+        """The recommended way to delete a SharePoint entity"""
+        qry = DeleteEntityQuery(self)
+        self.context.add_query(qry)
+        self.remove_from_parent_collection()
+        return self
+
     def update(self, *args):
-        """Updates the resource."""
+        """The recommended way to update a SharePoint entity"""
         qry = UpdateEntityQuery(self)
         self.context.add_query(qry)
         return self
