@@ -1,15 +1,15 @@
 from office365.runtime.client_result import ClientResult
 from office365.runtime.client_value_collection import ClientValueCollection
 from office365.runtime.queries.client_query import ClientQuery
-from office365.runtime.queries.delete_entity_query import DeleteEntityQuery
 from office365.runtime.queries.service_operation_query import ServiceOperationQuery
-from office365.runtime.queries.update_entity_query import UpdateEntityQuery
 from office365.runtime.resource_path import ResourcePath
 from office365.runtime.resource_path_service_operation import ResourcePathServiceOperation
 from office365.sharepoint.actions.getWebUrlFromPage import GetWebUrlFromPageUrlQuery
 from office365.sharepoint.alerts.alert_collection import AlertCollection
 from office365.sharepoint.changes.change_collection import ChangeCollection
+from office365.sharepoint.clientsidecomponent.types import SPClientSideComponentQueryResult
 from office365.sharepoint.contenttypes.content_type_collection import ContentTypeCollection
+from office365.sharepoint.eventreceivers.event_receiver_definition import EventReceiverDefinitionCollection
 from office365.sharepoint.fields.field_collection import FieldCollection
 from office365.sharepoint.files.file import File
 from office365.sharepoint.folders.folder import Folder
@@ -30,10 +30,14 @@ from office365.sharepoint.principal.user_collection import UserCollection
 from office365.sharepoint.recyclebin.recycleBinItemCollection import RecycleBinItemCollection
 from office365.sharepoint.sharing.externalSharingSiteOption import ExternalSharingSiteOption
 from office365.sharepoint.sharing.objectSharingSettings import ObjectSharingSettings
+from office365.sharepoint.sharing.sharingLinkData import SharingLinkData
 from office365.sharepoint.sharing.sharing_result import SharingResult
+from office365.sharepoint.sites.site_types import SiteCollectionCorporateCatalogAccessor
+from office365.sharepoint.tenant.administration.tenant_types import TenantCorporateCatalogAccessor
 from office365.sharepoint.ui.applicationpages.client_people_picker import (
     ClientPeoplePickerWebServiceInterface, ClientPeoplePickerQueryParameters
 )
+from office365.sharepoint.webparts.client_web_part_collection import ClientWebPartCollection
 from office365.sharepoint.webs.regional_settings import RegionalSettings
 from office365.sharepoint.webs.web_information_collection import WebInformationCollection
 from office365.sharepoint.webs.web_template_collection import WebTemplateCollection
@@ -54,6 +58,21 @@ class Web(SecurableObject):
         super(Web, self).__init__(context, resource_path)
         self._web_url = None
 
+    @staticmethod
+    def create_organization_sharing_link(context, url, is_edit_link):
+        """
+
+        :param office365.sharepoint.client_context.ClientContext context:
+        :param str url:
+        :param bool is_edit_link:
+        """
+        result = ClientResult(context)
+        params = {"url": url, "isEditLink": is_edit_link}
+        qry = ServiceOperationQuery(context.web, "CreateOrganizationSharingLink", None, params, None, result)
+        qry.static = True
+        context.add_query(qry)
+        return result
+
     @staticmethod
     def get_web_url_from_page_url(context, page_full_url):
         """Gets Web from page url
@@ -66,11 +85,26 @@ class Web(SecurableObject):
         return qry.return_type
 
     def get_all_client_side_components(self):
-        result = ClientResult(None)
+        result = ClientResult(self.context)
         qry = ServiceOperationQuery(self, "getAllClientSideComponents", None, None, None, result)
         self.context.add_query(qry)
         return result
 
+    def get_client_side_web_parts(self, project, includeErrors=False):
+        result = ClientValueCollection(SPClientSideComponentQueryResult)
+        params = {
+            "includeErrors": includeErrors,
+            "project": project
+        }
+        qry = ServiceOperationQuery(self, "getClientSideWebParts", None, params, None, result)
+        self.context.add_query(qry)
+        return result
+
+    def add_supported_ui_language(self, lcid):
+        qry = ServiceOperationQuery(self, "getSubWebsFilteredForCurrentUser", {"lcid": lcid}, None, None, None)
+        self.context.add_query(qry)
+        return self
+
     def get_sub_webs_filtered_for_current_user(self, query):
         """Returns a collection of objects that contain metadata about subsites of the current site (2) in which the
         current user is a member.
@@ -105,7 +139,7 @@ class Web(SecurableObject):
 
     def get_all_webs(self):
         """Returns a collection containing a flat list of all Web objects in the Web object."""
-        result = ClientResult(self.webs)
+        result = ClientResult(self.context, self.webs)
         qry = ClientQuery(self.context, self.webs, None, None, result)
         self.context.add_query(qry)
 
@@ -130,29 +164,21 @@ class Web(SecurableObject):
         return_list = List(self.context)
         self.lists.add_child(return_list)
         from office365.sharepoint.types.resource_path import ResourcePath as SPResPath
-        res_path = SPResPath(decoded_url)
-        qry = ServiceOperationQuery(self, "GetListUsingPath", res_path, None, None, return_list)
+        qry = ServiceOperationQuery(self, "GetListUsingPath", SPResPath(decoded_url), None, None, return_list)
         self.context.add_query(qry)
         return return_list
 
     def get_regional_datetime_schema(self):
-        return_type = ClientResult(str)
+        return_type = ClientResult(self.context)
         qry = ServiceOperationQuery(self, "GetRegionalDateTimeSchema", None, None, None, return_type)
         self.context.add_query(qry)
         return return_type
 
-    def update(self):
-        """Update a Web resource"""
-        qry = UpdateEntityQuery(self)
+    def get_sharing_link_data(self, linkUrl):
+        result = SharingLinkData()
+        qry = ServiceOperationQuery(self, "GetSharingLinkData", [linkUrl], None, None, result)
         self.context.add_query(qry)
-        return self
-
-    def delete_object(self):
-        """Delete a Web resource"""
-        qry = DeleteEntityQuery(self)
-        self.context.add_query(qry)
-        self.remove_from_parent_collection()
-        return self
+        return result
 
     @staticmethod
     def get_context_web_theme_data(context):
@@ -160,7 +186,7 @@ class Web(SecurableObject):
 
         :type context: office365.sharepoint.client_context.ClientContext
         """
-        result = ClientResult(str)
+        result = ClientResult(context)
         qry = ServiceOperationQuery(context.web, "GetContextWebThemeData", None, None, None, result)
         qry.static = True
         context.add_query(qry)
@@ -175,7 +201,7 @@ class Web(SecurableObject):
         string parameters
         :param office365.sharepoint.client_context.ClientContext context: client context
         """
-        result = ClientResult(bool)
+        result = ClientResult(context)
         payload = {
             "url": context.base_url + url,
             "isEditLink": is_edit_link
@@ -223,6 +249,17 @@ class Web(SecurableObject):
             ResourcePathServiceOperation("getFileByServerRelativeUrl", [url], self.resource_path)
         )
 
+    def get_file_by_server_relative_path(self, decoded_url):
+        """Returns the file object located at the specified server-relative path.
+        Prefer this method over get_folder_by_server_relative_url since it supports % and # symbols in names
+
+        :type decoded_url: str
+        """
+        return File(
+            self.context,
+            ResourcePathServiceOperation("getFileByServerRelativePath", {"DecodedUrl": decoded_url}, self.resource_path)
+        )
+
     def get_folder_by_server_relative_url(self, url):
         """Returns the folder object located at the specified server-relative URL.
         :type url: str
@@ -232,6 +269,19 @@ class Web(SecurableObject):
             ResourcePathServiceOperation("getFolderByServerRelativeUrl", [url], self.resource_path)
         )
 
+    def get_folder_by_server_relative_path(self, decoded_url):
+        """Returns the folder object located at the specified server-relative URL.
+        Prefer this method over get_folder_by_server_relative_url since it supports % and # symbols
+        Details: https://docs.microsoft.com/en-us/sharepoint/dev/solution-guidance/supporting-and-in-file-and-folder-with-the-resourcepath-api
+
+        :type decoded_url: str
+        """
+        params = {"DecodedUrl": decoded_url}
+        return File(
+            self.context,
+            ResourcePathServiceOperation("getFolderByServerRelativePath", params, self.resource_path)
+        )
+
     def ensure_folder_path(self, path):
         """
         Ensures a nested folder hierarchy exist
@@ -255,7 +305,7 @@ class Web(SecurableObject):
         """Gets the effective permissions that the specified user has within the current application scope.
         :type user_name: str
         """
-        result = ClientResult(BasePermissions())
+        result = ClientResult(self.context, BasePermissions())
         qry = ServiceOperationQuery(self, "GetUserEffectivePermissions", [user_name], None, None, result)
         self.context.add_query(qry)
         return result
@@ -265,7 +315,7 @@ class Web(SecurableObject):
 
         :type permission_mask: BasePermissions
         """
-        result = ClientResult(bool)
+        result = ClientResult(self.context)
         qry = ServiceOperationQuery(self, "doesUserHavePermissions", permission_mask, None, None, result)
         self.context.add_query(qry)
         return result
@@ -353,6 +403,24 @@ class Web(SecurableObject):
         self.context.add_query(qry)
         return return_type
 
+    def get_file_by_guest_url(self, guestUrl):
+        """
+        :type guestUrl: str
+        """
+        return_type = File(self.context)
+        qry = ServiceOperationQuery(self, "GetFileByGuestUrl", [guestUrl], None, None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
+    def get_folder_by_guest_url(self, guestUrl):
+        """
+        :type guestUrl: str
+        """
+        return_type = File(self.context)
+        qry = ServiceOperationQuery(self, "GetFolderByGuestUrl", [guestUrl], None, None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
     def share(self, user_principal_name,
               shareOption=ExternalSharingSiteOption.View,
               sendEmail=True, emailSubject=None, emailBody=None):
@@ -364,11 +432,11 @@ class Web(SecurableObject):
         :param bool sendEmail: A flag to determine if an email notification SHOULD be sent (if email is configured).
         :param str emailSubject: The email subject.
         :param str emailBody: The email subject.
-        :return: SharingResult
+        :rtype: SharingResult
         """
 
-        picker_result = ClientResult(str)
-        sharing_result = ClientResult(SharingResult(self.context))
+        picker_result = ClientResult(self.context)
+        sharing_result = ClientResult(self.context, SharingResult(self.context))
 
         def _picker_value_resolved(picker_value):
             picker_result.value = picker_value
@@ -391,15 +459,14 @@ class Web(SecurableObject):
         """
         Unshare a Web
 
-        :return: SharingResult
+        :rtype: SharingResult
         """
-        sharing_result = ClientResult(SharingResult(self.context))
+        result = ClientResult(self.context)
 
         def _web_initialized():
-            sharing_result.value = Web.unshare_object(self.context, self.url)
-
+            result.value = Web.unshare_object(self.context, self.url)
         self.ensure_property("Url", _web_initialized)
-        return sharing_result.value
+        return result.value
 
     @staticmethod
     def get_document_libraries(context, web_full_url):
@@ -412,7 +479,7 @@ class Web(SecurableObject):
         :param office365.sharepoint.client_context.ClientContext context: SharePoint context
         :param str web_full_url: The URL of the web.
         """
-        result = ClientValueCollection(DocumentLibraryInformation)
+        result = ClientResult(context, ClientValueCollection(DocumentLibraryInformation))
         payload = {
             "webFullUrl": web_full_url
         }
@@ -425,11 +492,11 @@ class Web(SecurableObject):
     def get_document_and_media_libraries(context, web_full_url, include_page_libraries):
         """
 
-        :param context:
+        :param office365.sharepoint.client_context.ClientContext context:
         :param str web_full_url:
         :param bool include_page_libraries:
         """
-        result = ClientValueCollection(DocumentLibraryInformation)
+        result = ClientResult(context, ClientValueCollection(DocumentLibraryInformation))
         payload = {
             "webFullUrl": web_full_url,
             "includePageLibraries": include_page_libraries
@@ -461,6 +528,19 @@ class Web(SecurableObject):
 
         context.after_execute(_group_resolved)
 
+    @staticmethod
+    def get_sharing_link_kind(context, fileUrl):
+        """
+
+        :param office365.sharepoint.client_context.ClientContext context:
+        :param str fileUrl:
+        """
+        result = ClientResult(context)
+        qry = ServiceOperationQuery(context.web, "GetSharingLinkKind", None, {"fileUrl": fileUrl}, None, result)
+        qry.static = True
+        context.add_query(qry)
+        return result
+
     @staticmethod
     def share_object(context, url, peoplePickerInput,
                      roleValue=None,
@@ -562,96 +642,74 @@ class Web(SecurableObject):
             if self.is_property_available('Url'):
                 parent_web_url = self.properties['Url']
             return WebCollection(self.context,
-                                 ResourcePath("webs", self.resource_path),
-                                 parent_web_url)
+                                 ResourcePath("webs", self.resource_path), parent_web_url)
 
     @property
     def folders(self):
         """Get folder resources"""
-        if self.is_property_available('Folders'):
-            return self.properties['Folders']
-        else:
-            return FolderCollection(self.context, ResourcePath("folders", self.resource_path))
+        return self.properties.get('Folders',
+                                   FolderCollection(self.context, ResourcePath("folders", self.resource_path)))
 
     @property
     def lists(self):
         """Get web list collection"""
-        if self.is_property_available('Lists'):
-            return self.properties['Lists']
-        else:
-            return ListCollection(self.context, ResourcePath("lists", self.resource_path))
+        return self.properties.get('Lists',
+                                   ListCollection(self.context, ResourcePath("lists", self.resource_path)))
 
     @property
     def siteUsers(self):
         """Get site users"""
-        if self.is_property_available('SiteUsers'):
-            return self.properties['SiteUsers']
-        else:
-            return UserCollection(self.context, ResourcePath("siteUsers", self.resource_path))
+        return self.properties.get('SiteUsers',
+                                   UserCollection(self.context, ResourcePath("siteUsers", self.resource_path)))
 
     @property
     def siteGroups(self):
         """Gets the collection of groups for the site collection."""
-        if self.is_property_available('SiteGroups'):
-            return self.properties['SiteGroups']
-        else:
-            return GroupCollection(self.context, ResourcePath("siteGroups", self.resource_path))
+        return self.properties.get('SiteGroups',
+                                   GroupCollection(self.context, ResourcePath("siteGroups", self.resource_path)))
 
     @property
     def current_user(self):
         """Gets the current user."""
-        if self.is_property_available('CurrentUser'):
-            return self.properties['CurrentUser']
-        else:
-            return User(self.context, ResourcePath("CurrentUser", self.resource_path))
+        return self.properties.get('CurrentUser',
+                                   User(self.context, ResourcePath("CurrentUser", self.resource_path)))
 
     @property
     def parentWeb(self):
         """Gets the parent website of the specified website."""
-        if self.is_property_available('ParentWeb'):
-            return self.properties['ParentWeb']
-        else:
-            return User(self.context, ResourcePath("ParentWeb", self.resource_path))
+        return self.properties.get('ParentWeb',
+                                   Web(self.context, ResourcePath("ParentWeb", self.resource_path)))
 
     @property
     def associatedVisitorGroup(self):
         """Gets or sets the associated visitor group of the Web site."""
-        if self.is_property_available('AssociatedVisitorGroup'):
-            return self.properties['AssociatedVisitorGroup']
-        else:
-            return Group(self.context, ResourcePath("AssociatedVisitorGroup", self.resource_path))
+        return self.properties.get('AssociatedVisitorGroup',
+                                   Group(self.context, ResourcePath("AssociatedVisitorGroup", self.resource_path)))
 
     @property
     def associatedOwnerGroup(self):
         """Gets or sets the associated owner group of the Web site."""
-        if self.is_property_available('AssociatedOwnerGroup'):
-            return self.properties['AssociatedOwnerGroup']
-        else:
-            return Group(self.context, ResourcePath("AssociatedOwnerGroup", self.resource_path))
+        return self.properties.get('AssociatedOwnerGroup',
+                                   Group(self.context, ResourcePath("AssociatedOwnerGroup", self.resource_path)))
 
     @property
     def associatedMemberGroup(self):
         """Gets or sets the group of users who have been given contribute permissions to the Web site."""
-        if self.is_property_available('AssociatedMemberGroup'):
-            return self.properties['AssociatedMemberGroup']
-        else:
-            return Group(self.context, ResourcePath("AssociatedMemberGroup", self.resource_path))
+        return self.properties.get('AssociatedMemberGroup',
+                                   Group(self.context, ResourcePath("AssociatedMemberGroup", self.resource_path)))
 
     @property
     def fields(self):
         """Specifies the collection of all the fields (2) in the site (2)."""
-        if self.is_property_available('Fields'):
-            return self.properties['Fields']
-        else:
-            return FieldCollection(self.context, ResourcePath("Fields", self.resource_path))
+        return self.properties.get('Fields',
+                                   FieldCollection(self.context, ResourcePath("Fields", self.resource_path)))
 
     @property
     def content_types(self):
         """Gets the collection of content types for the Web site."""
-        if self.is_property_available('ContentTypes'):
-            return self.properties['ContentTypes']
-        else:
-            return ContentTypeCollection(self.context, ResourcePath("ContentTypes", self.resource_path))
+        return self.properties.get('ContentTypes',
+                                   ContentTypeCollection(self.context,
+                                                         ResourcePath("ContentTypes", self.resource_path), self))
 
     @property
     def role_definitions(self):
@@ -661,6 +719,34 @@ class Web(SecurableObject):
                                                             ResourcePath("RoleDefinitions",
                                                                          self.resource_path)))
 
+    @property
+    def event_receivers(self):
+        """Get Event receivers"""
+        return self.properties.get('EventReceivers',
+                                   EventReceiverDefinitionCollection(self.context,
+                                                                     ResourcePath("eventReceivers", self.resource_path),
+                                                                     self))
+
+    @property
+    def client_web_parts(self):
+        """Client Web Parts"""
+        return self.properties.get('ClientWebParts',
+                                   ClientWebPartCollection(self.context,
+                                                           ResourcePath("ClientWebParts", self.resource_path)))
+
+    @property
+    def tenant_app_catalog(self):
+        return self.properties.get('TenantAppCatalog',
+                                   TenantCorporateCatalogAccessor(self.context,
+                                                                  ResourcePath("TenantAppCatalog", self.resource_path)))
+
+    @property
+    def site_collection_app_catalog(self):
+        return self.properties.get('SiteCollectionAppCatalog',
+                                   SiteCollectionCorporateCatalogAccessor(self.context,
+                                                                          ResourcePath("SiteCollectionAppCatalog",
+                                                                                       self.resource_path)))
+
     @property
     def url(self):
         """Gets the absolute URL for the website.
@@ -668,6 +754,27 @@ class Web(SecurableObject):
         """
         return self.properties.get('Url', None)
 
+    @property
+    def quick_launch_enabled(self):
+        """Gets a value that specifies whether the Quick Launch area is enabled on the site.
+        :rtype: bool or None
+        """
+        return self.properties.get('QuickLaunchEnabled', None)
+
+    @quick_launch_enabled.setter
+    def quick_launch_enabled(self, value):
+        """Sets a value that specifies whether the Quick Launch area is enabled on the site.
+        :type value: bool
+        """
+        self.set_property('QuickLaunchEnabled', value)
+
+    @property
+    def site_logo_url(self):
+        """Gets a value that specifies Site logo url.
+        :rtype: str or None
+        """
+        return self.properties.get('SiteLogoUrl', None)
+
     @property
     def list_templates(self):
         """Gets a value that specifies the collection of list definitions and list templates available for creating
@@ -686,10 +793,8 @@ class Web(SecurableObject):
     @property
     def regional_settings(self):
         """Gets the regional settings that are currently implemented on the website."""
-        if self.is_property_available('RegionalSettings'):
-            return self.properties['RegionalSettings']
-        else:
-            return RegionalSettings(self.context, ResourcePath("RegionalSettings", self.resource_path))
+        return self.properties.get('RegionalSettings',
+                                   RegionalSettings(self.context, ResourcePath("RegionalSettings", self.resource_path)))
 
     @property
     def recycle_bin(self):
@@ -722,6 +827,28 @@ class Web(SecurableObject):
                                    FieldCollection(self.context,
                                                    ResourcePath("AvailableFields", self.resource_path)))
 
+    @property
+    def site_user_info_list(self):
+        return self.properties.get('SiteUserInfoList',
+                                   List(self.context, ResourcePath("SiteUserInfoList", self.resource_path)))
+
+    @property
+    def welcome_page(self):
+        return self.properties.get('WelcomePage', None)
+
+    @property
+    def supported_ui_language_ids(self):
+        """Specifies the language code identifiers (LCIDs) of the languages that are enabled for the site."""
+        return self.properties.get('SupportedUILanguageIds', ClientValueCollection(int))
+
+    @property
+    def ui_version(self):
+        """
+        Gets or sets the user interface (UI) version of the Web site.
+        :rtype: int or None
+        """
+        return self.properties.get('UIVersion', None)
+
     def get_property(self, name):
         if name == "ContentTypes":
             return self.content_types
