@@ -1,3 +1,4 @@
+from tests import test_user_principal_name
 from tests.sharepoint.sharepoint_case import SPTestCase
 
 from office365.sharepoint.principal.group import Group
@@ -9,7 +10,7 @@ class TestSharePointGroup(SPTestCase):
     @classmethod
     def setUpClass(cls):
         super(TestSharePointGroup, cls).setUpClass()
-        cls.target_user = cls.client.web.ensure_user(cls.test_user_name).execute_query()
+        cls.target_user = cls.client.web.ensure_user(test_user_principal_name).execute_query()
         cls.target_group = cls.client.web.associatedMemberGroup
 
     def test1_get_current_user_groups(self):
